@@ -2,7 +2,7 @@
 
 #BEGIN {$^W=1};
 
-use Gimp::Feature qw(:pdl);
+use Gimp::Feature qw(pdl);
 use Gimp;
 use Gimp::Fu;
 use Gimp::PDL;
